#!/usr/bin/env python
# encoding: utf-8
# 
# nengo.py - convert between Japanese regnal year and A.D. year.
# 年号西暦変換。


from bisect import bisect_right


NAME_TABLES = [
    [
        (645, u'大化'),
        (650, u'白雉'),
        (655, u'（斉明）'),
        (662, u'（天智）'),
        (672, u'（天武）'),
        (686, u'朱鳥'),
        (687, u'（持統）'),
        (697, u'（文武）'),
        (701, u'大宝'),
        (704, u'慶雲'),
        (708, u'和銅'),
        (715, u'霊亀'),
        (717, u'養老'),
        (724, u'神亀'),
        (729, u'天平'),
        (749, u'天平感宝'),
        (749, u'天平勝宝'),
        (757, u'天平宝字'),
        (765, u'天平神護'),
        (767, u'神護景雲'),
        (770, u'宝亀'),
        (781, u'天応'),
        (782, u'延暦'),
        (806, u'大同'),
        (810, u'弘仁'),
        (824, u'天長'),
        (834, u'承和'),
        (848, u'嘉祥'),
        (851, u'仁寿'),
        (854, u'斉衡'),
        (857, u'天安'),
        (859, u'貞観'),
        (877, u'元慶'),
        (885, u'仁和'),
        (889, u'寛平'),
        (898, u'昌泰'),
        (901, u'延喜'),
        (923, u'延長'),
        (931, u'承平'),
        (938, u'天慶'),
        (947, u'天暦'),
        (957, u'天徳'),
        (961, u'応和'),
        (964, u'康保'),
        (968, u'安和'),
        (970, u'天禄'),
        (973, u'天延'),
        (976, u'貞元'),
        (978, u'天元'),
        (983, u'永観'),
        (985, u'寛和'),
        (987, u'永延'),
        (989, u'永祚'),
        (990, u'正暦'),
        (995, u'長徳'),
        (999, u'長保'),
        (1004, u'寛弘'),
        (1012, u'長和'),
        (1017, u'寛仁'),
        (1021, u'治安'),
        (1024, u'万寿'),
        (1028, u'長元'),
        (1037, u'長暦'),
        (1040, u'長久'),
        (1044, u'寛徳'),
        (1046, u'永承'),
        (1053, u'天喜'),
        (1058, u'康平'),
        (1065, u'治暦'),
        (1069, u'延久'),
        (1074, u'承保'),
        (1077, u'承暦'),
        (1081, u'永保'),
        (1084, u'応徳'),
        (1087, u'寛治'),
        (1094, u'嘉保'),
        (1096, u'永長'),
        (1097, u'承徳'),
        (1099, u'康和'),
        (1104, u'長治'),
        (1106, u'嘉承'),
        (1108, u'天仁'),
        (1110, u'天永'),
        (1113, u'永久'),
        (1118, u'元永'),
        (1120, u'保安'),
        (1124, u'天治'),
        (1126, u'大治'),
        (1131, u'天承'),
        (1132, u'長承'),
        (1135, u'保延'),
        (1141, u'永治'),
        (1142, u'康治'),
        (1144, u'天養'),
        (1145, u'久安'),
        (1151, u'仁平'),
        (1154, u'久寿'),
        (1156, u'保元'),
        (1159, u'平治'),
        (1160, u'永暦'),
        (1161, u'応保'),
        (1163, u'長寛'),
        (1165, u'永万'),
        (1166, u'仁安'),
        (1169, u'嘉応'),
        (1171, u'承安'),
        (1175, u'安元'),
        (1177, u'治承'),
        (1181, u'養和'),
        (1182, u'寿永'),
        (1184, u'元暦'),
        (1185, u'文治'),
        (1190, u'建久'),
        (1199, u'正治'),
        (1201, u'建仁'),
        (1204, u'元久'),
        (1206, u'建永'),
        (1207, u'承元'),
        (1211, u'建暦'),
        (1213, u'建保'),
        (1219, u'承久'),
        (1222, u'貞応'),
        (1224, u'元仁'),
        (1225, u'嘉禄'),
        (1227, u'安貞'),
        (1229, u'寛喜'),
        (1232, u'貞永'),
        (1233, u'天福'),
        (1234, u'文暦'),
        (1235, u'嘉禎'),
        (1238, u'暦仁'),
        (1239, u'延応'),
        (1240, u'仁治'),
        (1243, u'寛元'),
        (1247, u'宝治'),
        (1249, u'建長'),
        (1256, u'康元'),
        (1257, u'正嘉'),
        (1259, u'正元'),
        (1260, u'文応'),
        (1261, u'弘長'),
        (1264, u'文永'),
        (1275, u'建治'),
        (1278, u'弘安'),
        (1288, u'正応'),
        (1293, u'永仁'),
        (1299, u'正安'),
        (1302, u'乾元'),
        (1303, u'嘉元'),
        (1306, u'徳治'),
        (1308, u'延慶'),
        (1311, u'応長'),
        (1312, u'正和'),
        (1317, u'文保'),
        (1319, u'元応'),
        (1321, u'元亨'),
        (1324, u'正中'),
        (1326, u'嘉暦'),
        (1329, None),
    ],
    [   # 南朝元徳以降
        (1331, u'元弘'),
        (1334, None),   # 建武
    ],
    [
        (1329, u'元徳'),
        (1332, None),   # 北朝は1332年まで元徳。
    ],
    [   # 北朝元徳以降
        (1332, u'正慶'),
        (1334, None),   # 建武
    ],
    [   # 南朝建武以降
        (1336, u'延元'),
        (1340, u'興国'),
        (1346, u'正平'),
        (1370, u'建徳'),
        (1372, u'文中'),
        (1375, u'天授'),
        (1381, u'弘和'),
        (1384, u'元中'),
        (1392, None),   # 元中は1392年まで。
    ],
    [
        (1334, u'建武'),
        (1338, None),   # 北朝は1338年まで建武。
    ],
    [   # 北朝建武以降
        (1338, u'暦応'),
        (1342, u'康永'),
        (1345, u'貞和'),
        (1350, u'観応'),
        (1352, u'文和'),
        (1356, u'延文'),
        (1361, u'康安'),
        (1362, u'貞治'),
        (1368, u'応安'),
        (1375, u'永和'),
        (1379, u'康暦'),
        (1381, u'永徳'),
        (1384, u'至徳'),
        (1387, u'嘉慶'),
        (1389, u'康応'),
        (1390, u'明徳'),
        (1394, u'応永'),
        (1428, u'正長'),
        (1429, u'永享'),
        (1441, u'嘉吉'),
        (1444, u'文安'),
        (1449, u'宝徳'),
        (1452, u'享徳'),
        (1455, u'康正'),
        (1457, u'長禄'),
        (1460, u'寛正'),
        (1466, u'文正'),
        (1573, u'天正'),
        (1592, u'文禄'),
        (1596, u'慶長'),
        (1615, u'元和'),
        (1624, u'寛永'),
        (1644, u'正保'),
        (1648, u'慶安'),
        (1652, u'承応'),
        (1655, u'明暦'),
        (1658, u'万治'),
        (1661, u'寛文'),
        (1673, u'延宝'),
        (1681, u'天和'),
        (1684, u'貞享'),
        (1688, u'元禄'),
        (1704, u'宝永'),
        (1711, u'正徳'),
        (1716, u'享保'),
        (1736, u'元文'),
        (1741, u'寛保'),
        (1744, u'延享'),
        (1748, u'寛延'),
        (1751, u'宝暦'),
        (1764, u'明和'),
        (1772, u'安永'),
        (1781, u'天明'),
        (1789, u'寛政'),
        (1801, u'享和'),
        (1804, u'文化'),
        (1818, u'文政'),
        (1830, u'天保'),
        (1844, u'弘化'),
        (1848, u'嘉永'),
        (1854, u'安政'),
        (1860, u'万延'),
        (1861, u'文久'),
        (1864, u'元治'),
        (1865, u'慶応'),
        (1868, u'明治'),
        (1912, u'大正'),
        (1926, u'昭和'),
        (1989, u'平成'),
        (99999, None),
    ],
]


YEAR_DICT = {
    # 年号-開始年の辞書
    u'大化': 645,
    u'白雉': 650,
    u'（斉明）': 655,
    u'（天智）': 662,
    u'（天武）': 672,
    u'朱鳥': 686,
    u'（持統）': 687,
    u'（文武）': 697,
    u'大宝': 701,
    u'慶雲': 704,
    u'和銅': 708,
    u'霊亀': 715,
    u'養老': 717,
    u'神亀': 724,
    u'天平': 729,
    u'天平感宝': 749,
    u'天平勝宝': 749,
    u'天平宝字': 757,
    u'天平神護': 765,
    u'神護景雲': 767,
    u'宝亀': 770,
    u'天応': 781,
    u'延暦': 782,
    u'大同': 806,
    u'弘仁': 810,
    u'天長': 824,
    u'承和': 834,
    u'嘉祥': 848,
    u'仁寿': 851,
    u'斉衡': 854,
    u'天安': 857,
    u'貞観': 859,
    u'元慶': 877,
    u'仁和': 885,
    u'寛平': 889,
    u'昌泰': 898,
    u'延喜': 901,
    u'延長': 923,
    u'承平': 931,
    u'天慶': 938,
    u'天暦': 947,
    u'天徳': 957,
    u'応和': 961,
    u'康保': 964,
    u'安和': 968,
    u'天禄': 970,
    u'天延': 973,
    u'貞元': 976,
    u'天元': 978,
    u'永観': 983,
    u'寛和': 985,
    u'永延': 987,
    u'永祚': 989,
    u'正暦': 990,
    u'長徳': 995,
    u'長保': 999,
    u'寛弘': 1004,
    u'長和': 1012,
    u'寛仁': 1017,
    u'治安': 1021,
    u'万寿': 1024,
    u'長元': 1028,
    u'長暦': 1037,
    u'長久': 1040,
    u'寛徳': 1044,
    u'永承': 1046,
    u'天喜': 1053,
    u'康平': 1058,
    u'治暦': 1065,
    u'延久': 1069,
    u'承保': 1074,
    u'承暦': 1077,
    u'永保': 1081,
    u'応徳': 1084,
    u'寛治': 1087,
    u'嘉保': 1094,
    u'永長': 1096,
    u'承徳': 1097,
    u'康和': 1099,
    u'長治': 1104,
    u'嘉承': 1106,
    u'天仁': 1108,
    u'天永': 1110,
    u'永久': 1113,
    u'元永': 1118,
    u'保安': 1120,
    u'天治': 1124,
    u'大治': 1126,
    u'天承': 1131,
    u'長承': 1132,
    u'保延': 1135,
    u'永治': 1141,
    u'康治': 1142,
    u'天養': 1144,
    u'久安': 1145,
    u'仁平': 1151,
    u'久寿': 1154,
    u'保元': 1156,
    u'平治': 1159,
    u'永暦': 1160,
    u'応保': 1161,
    u'長寛': 1163,
    u'永万': 1165,
    u'仁安': 1166,
    u'嘉応': 1169,
    u'承安': 1171,
    u'安元': 1175,
    u'治承': 1177,
    u'養和': 1181,
    u'寿永': 1182,
    u'元暦': 1184,
    u'文治': 1185,
    u'建久': 1190,
    u'正治': 1199,
    u'建仁': 1201,
    u'元久': 1204,
    u'建永': 1206,
    u'承元': 1207,
    u'建暦': 1211,
    u'建保': 1213,
    u'承久': 1219,
    u'貞応': 1222,
    u'元仁': 1224,
    u'嘉禄': 1225,
    u'安貞': 1227,
    u'寛喜': 1229,
    u'貞永': 1232,
    u'天福': 1233,
    u'文暦': 1234,
    u'嘉禎': 1235,
    u'暦仁': 1238,
    u'延応': 1239,
    u'仁治': 1240,
    u'寛元': 1243,
    u'宝治': 1247,
    u'建長': 1249,
    u'康元': 1256,
    u'正嘉': 1257,
    u'正元': 1259,
    u'文応': 1260,
    u'弘長': 1261,
    u'文永': 1264,
    u'建治': 1275,
    u'弘安': 1278,
    u'正応': 1288,
    u'永仁': 1293,
    u'正安': 1299,
    u'乾元': 1302,
    u'嘉元': 1303,
    u'徳治': 1306,
    u'延慶': 1308,
    u'応長': 1311,
    u'正和': 1312,
    u'文保': 1317,
    u'元応': 1319,
    u'元亨': 1321,
    u'正中': 1324,
    u'嘉暦': 1326,
    u'元徳': 1329,
    u'元弘': 1331,
    u'正慶': 1332,
    u'建武': 1334,
    u'延元': 1336,
    u'暦応': 1338,
    u'興国': 1340,
    u'康永': 1342,
    u'貞和': 1345,
    u'正平': 1346,
    u'観応': 1350,
    u'文和': 1352,
    u'延文': 1356,
    u'康安': 1361,
    u'貞治': 1362,
    u'応安': 1368,
    u'建徳': 1370,
    u'文中': 1372,
    u'天授': 1375,
    u'永和': 1375,
    u'康暦': 1379,
    u'弘和': 1381,
    u'永徳': 1381,
    u'元中': 1384,
    u'至徳': 1384,
    u'嘉慶': 1387,
    u'康応': 1389,
    u'明徳': 1390,
    u'応永': 1394,
    u'正長': 1428,
    u'永享': 1429,
    u'嘉吉': 1441,
    u'文安': 1444,
    u'宝徳': 1449,
    u'享徳': 1452,
    u'康正': 1455,
    u'長禄': 1457,
    u'寛正': 1460,
    u'文正': 1466,
    u'天正': 1573,
    u'文禄': 1592,
    u'慶長': 1596,
    u'元和': 1615,
    u'寛永': 1624,
    u'正保': 1644,
    u'慶安': 1648,
    u'承応': 1652,
    u'明暦': 1655,
    u'万治': 1658,
    u'寛文': 1661,
    u'延宝': 1673,
    u'天和': 1681,
    u'貞享': 1684,
    u'元禄': 1688,
    u'宝永': 1704,
    u'正徳': 1711,
    u'享保': 1716,
    u'元文': 1736,
    u'寛保': 1741,
    u'延享': 1744,
    u'寛延': 1748,
    u'宝暦': 1751,
    u'明和': 1764,
    u'安永': 1772,
    u'天明': 1781,
    u'寛政': 1789,
    u'享和': 1801,
    u'文化': 1804,
    u'文政': 1818,
    u'天保': 1830,
    u'弘化': 1844,
    u'嘉永': 1848,
    u'安政': 1854,
    u'万延': 1860,
    u'文久': 1861,
    u'元治': 1864,
    u'慶応': 1865,
    u'明治': 1868,
    u'大正': 1912,
    u'昭和': 1926,
    u'平成': 1989,
}

# 干支
STEMS = u'甲乙丙丁戊己庚辛壬癸'
BRANCHES = u'子丑寅卯辰巳午未申酉戌亥'


def cycle_name(n):
    # 干支文字列を返す。
    return STEMS[n % len(STEMS)] + BRANCHES[n % len(BRANCHES)]


def cycle_of_year(year):
    # 西暦からその年の干支を返す。
    return (year + 56) % 60


def nengo(year):
    # 西暦から対応する年号と年番のペアのリストを返す。
    if year > 10000: return []
    L = []
    for table in NAME_TABLES:
        i = bisect_right(table, (year, u'\uffff')) - 1
        n, name = table[i]
        if name:
            L.append((name, year-n+1))
    return L


def year_from_nengo(name, number):
    # 年号と年番から西暦を返す。
    return YEAR_DICT[name] + number - 1


if __name__ == '__main__':
    
    import locale
    import re
    import sys
    from optparse import OptionParser
    
    locale.setlocale(locale.LC_ALL, '')
    encoding = locale.getlocale()[1]
    
    def output(s):
        sys.stdout.write(s.encode(encoding))
    
    parser = OptionParser(usage='nengo.py [options] year[-year]')
    opts, args = parser.parse_args()
    if len(args) != 1:
        parser.print_help()
        sys.exit(-1)
    
    rx = re.match(r'(\d+)(?:\-(\d+))?', args[0])
    if not rx:
        parser.print_help()
        sys.exit(-1)
    start = int(rx.group(1))
    end = int(rx.group(2) or start) + 1
    
    for year in xrange(start, end):
        output('%d; ' % year)
        output(', '.join(['%s%d' % (s, n) for s, n in nengo(year)]))
        output('; %s' % cycle_name(cycle_of_year(year)))
        output('\n')

